#include "../SDK/foobar2000.h"
#include "../columns_ui-sdk/ui_extension.h"
#include "../SDK/service.h"
#include "resource.h"
#include "../shared/shared.h"
#pragma comment(lib,"../shared/shared.lib")
using namespace pfc;
using namespace ui_extension;

class uie_tabs_panel;

class uieh_tabs : public ui_extension::window_host_with_control
{
private:
	class tabbed_ext
	{
	public:
		ui_extension::window_ptr ptr;
		string8 caption;
		tabbed_ext(ui_extension::window_ptr & p, const char * name = 0);
		~tabbed_ext();
		bool has_default_name();
	};

	pfc::ptr_list_t<tabbed_ext> exts;
	pfc::ptr_list_t<tabbed_ext> pre_exts;

	HWND wnd;
	HWND tab_wnd;
	HWND active_wnd;
	unsigned selected_tab;
	unsigned selected_tab_cfg;
	unsigned tab_orient;
    bool hide_tab;
    unsigned int limit_min_height;
	uie_tabs_panel * parent_ext;
	HFONT font;
    static pfc::ptr_list_t<uieh_tabs> g_list_tabs;

//	cfg_var::read_config_helper * config_helper;

public:
	enum tab_orientation
	{
		TABS_ON_TOP,
		TABS_ON_BOTTOM,
		TABS_ON_LEFT,
		TABS_ON_RIGHT
	};
	enum tab_style
	{
		TABS_TAB,
		TABS_BUTTON
	};
    static pfc::ptr_list_t<uieh_tabs> & g_get_list_tabs() { return g_list_tabs; }
    const pfc::ptr_list_t<tabbed_ext> & get_exts() { return exts; }

	uieh_tabs(uie_tabs_panel * parent);
	uieh_tabs();
	~uieh_tabs();

	static const GUID host_guid;
	virtual const GUID & get_host_guid() const { return host_guid; }

	virtual void get_name(string_base & out) const { out = "Panel stack"; }
	virtual bool is_available() const { return get_wnd()!=0? true : false; }
	virtual unsigned get_supported_types() const {
		return uie::type_splitter|uie::type_playlist|ui_extension::type_panel|ui_extension::type_toolbar|ui_extension::type_layout; }

	virtual void insert_extension (const GUID & in, unsigned height, unsigned width);
	virtual void insert_extension (ui_extension::window_ptr & p_ext, unsigned height, unsigned width);
	void insert_extension (ui_extension::window_ptr & p_ext, const char * title = 0);
	void insert_extension (const GUID & in, const char * title = 0);
	
	void insert_extension (const GUID & in, stream_reader * p_reader, t_size p_size, abort_callback & p_abort, const char * title = 0);

	virtual void on_size_limit_change(HWND wnd, unsigned flags) {}
	virtual unsigned is_resize_supported(HWND wnd);
	virtual bool request_resize(HWND wnd, unsigned flags, unsigned width, unsigned height);

	virtual bool override_status_text_create(service_ptr_t<ui_status_text_override> & p_out);
	virtual bool get_keyboard_shortcuts_enabled() const;

	virtual bool is_visible(HWND wnd) const;
	virtual bool is_visibility_modifiable(HWND wnd, bool desired_visibility) const;
	virtual bool set_window_visibility(HWND wnd, bool visibility);

	virtual void relinquish_ownership(HWND wnd);
	unsigned int is_resize_supported(HWND) const { return uie::size_width|uie::size_height;}
	virtual HWND create_window(HWND parent);
	void destroy_window();
	HWND get_wnd() const { return wnd; }
    HWND get_tab_wnd() const { return tab_wnd; }

	void do_context_menu(POINT pt, unsigned target_tab);

	virtual void get_config(stream_writer * p_writer, abort_callback & p_abort);
	virtual void set_config(stream_reader * p_reader, t_size p_size, abort_callback & p_abort);

	static BOOL CALLBACK dialog_proc(HWND wnd, UINT msg, WPARAM wp, LPARAM lp);
protected:
	virtual BOOL CALLBACK on_message(HWND wnd, UINT msg, WPARAM wp, LPARAM lp);
private:

	inline tab_orientation get_tab_bar_orientation()
	{
		if (uGetWindowLong(tab_wnd,GWL_STYLE) & TCS_BOTTOM)
			return TABS_ON_BOTTOM;
		return TABS_ON_TOP;
	}
	
	inline tab_style get_tab_style()
	{
		if (uGetWindowLong(tab_wnd,GWL_STYLE) & TCS_BUTTONS)
			return TABS_BUTTON;
		return TABS_TAB;
	}
	void set_tab_bar_orientation(tab_orientation o);
	void set_tab_style(tab_style s);
	HWND get_child_window(unsigned idx);
	RECT get_display_area();
	void size_child_window(HWND wnd);
public:
	void on_tab_sel_change();
    void on_tab_sel_change(unsigned int p_index);
    bool get_hide_tab() { return hide_tab; }
    void toggle_hide_tab();
    void change_limit_min_height();
	void swap_tabs(unsigned i1, unsigned i2);
	ui_extension::window_ptr& get_child_extension(unsigned idx) const ;
	void delete_tab(unsigned idx, bool destroy_window = true);
	unsigned get_child_count()const{ return exts.get_count() ? exts.get_count() : pre_exts.get_count();}
	void get_child_caption(unsigned idx, string8 &p_out)
	{   if(exts.get_count()) p_out = exts[idx]->caption;
		else p_out = pre_exts[idx]->caption; }
	bool has_default_name(unsigned idx){ return exts.get_count() ? exts[idx]->has_default_name() : pre_exts[idx]->has_default_name();}
private:
	void rename_tab(unsigned idx, const char * new_name);
	void do_tab_rename(unsigned idx);
	static BOOL CALLBACK rename_proc(HWND wnd, UINT msg, WPARAM wp, LPARAM lp);
    static BOOL CALLBACK set_limit_min_height_proc(HWND wnd, UINT msg, WPARAM wp, LPARAM lp);
};

class uie_tabs_panel : public ui_extension::splitter_window
{
private:
	ui_extension::window_host_ptr p_host;
	service_ptr_t<uieh_tabs> p_containing_host;
public:

	uie_tabs_panel();
	~uie_tabs_panel();

	static const GUID extension_guid;
	virtual const GUID & get_extension_guid() const { return extension_guid; }

	virtual unsigned get_type() const { return uie::type_splitter|ui_extension::type_panel|ui_extension::type_layout; }
	virtual void get_category(string_base & out) const { out = "Layout"; }
	virtual void get_name(string_base & out) const { out = "Tabbed panel stack mod"; }
	virtual bool get_short_name(string_base & out) const { out = "Panel stack mod"; return true;};
	virtual bool get_description(string_base & out) const
	{
		out = "Extension host supporting tabs.";
		return true;
	}

	virtual bool is_available(const ui_extension::window_host_ptr &p_target_host) const
	{
		if (p_target_host->get_host_guid() == uieh_tabs::host_guid)
			return false;
		return true;
	}
	virtual HWND create_or_transfer_window(HWND,const ui_extension::window_host_ptr &, 
		const ui_helpers::window_position_t &);
	virtual void destroy_window();
	virtual HWND get_wnd() const;
	ui_extension::window_host_ptr & get_host() { return p_host; }

	virtual void set_config(stream_reader * p_reader, t_size p_size, abort_callback & p_abort);
	virtual void get_config(stream_writer * p_writer, abort_callback & p_abort) const;

	virtual void insert_panel(t_size index, const splitter_item_t * p_item);	
	virtual void remove_panel(t_size index);
	virtual void replace_panel(t_size index, const splitter_item_t * p_item);
	virtual t_size get_panel_count()const;

protected:
	virtual splitter_item_t * get_panel(t_size index)const;
};

